@@ -553,8 +553,11 @@ static struct kgem_bo *upload(struct sna *sna,
 		    pixmap->usage_hint == 0 &&
 		    channel->width  == pixmap->drawable.width &&
 		    channel->height == pixmap->drawable.height) {
+			DBG(("%s: adding upload cache to pixmap=%ld\n",
+			     __FUNCTION__, pixmap->drawable.serialNumber));
 			assert(priv->gpu_damage == NULL);
 			assert(priv->gpu_bo == NULL);
+			assert(bo->proxy != NULL);
 			kgem_proxy_bo_attach(bo, &priv->gpu_bo);
 		}
 	}
@@ -1210,8 +1213,11 @@ sna_render_picture_extract(struct sna *sna,
 			if (priv != NULL && bo != NULL &&
 			    box.x2 - box.x1 == pixmap->drawable.width &&
 			    box.y2 - box.y1 == pixmap->drawable.height) {
+				DBG(("%s: adding upload cache to pixmap=%ld\n",
+				     __FUNCTION__, pixmap->drawable.serialNumber));
 				assert(priv->gpu_damage == NULL);
 				assert(priv->gpu_bo == NULL);
+				assert(bo->proxy != NULL);
 				kgem_proxy_bo_attach(bo, &priv->gpu_bo);
 			}
 		}
@@ -1330,7 +1336,7 @@ sna_render_picture_convolve(struct sna *sna,
 
 	ValidatePicture(tmp);
 
-	bo = sna_pixmap_get_bo(pixmap);
+	bo = __sna_pixmap_get_bo(pixmap);
 	if (!sna->render.clear(sna, pixmap, bo)) {
 		FreePicture(tmp, 0);
 		return 0;
@@ -1444,7 +1450,7 @@ sna_render_picture_flatten(struct sna *sna,
 	channel->scale[1] = 1.f / h;
 	channel->offset[0] = -dst_x;
 	channel->offset[1] = -dst_y;
-	channel->bo = kgem_bo_reference(sna_pixmap_get_bo(pixmap));
+	channel->bo = kgem_bo_reference(__sna_pixmap_get_bo(pixmap));
 	FreePicture(tmp, 0);
 
 	return 1;
@@ -1480,7 +1486,7 @@ sna_render_picture_approximate_gradient(struct sna *sna,
 		return -1;
 	}
 
-	channel->is_opaque = sna_gradient_is_opaque(picture->pSourcePict);
+	channel->is_opaque = sna_gradient_is_opaque((PictGradient*)picture->pSourcePict);
 	channel->pict_format =
 		channel->is_opaque ? PIXMAN_x8r8g8b8 : PIXMAN_a8r8g8b8;
 	DBG(("%s: gradient is opaque? %d, selecting format %08x\n",
@@ -1811,7 +1817,7 @@ sna_render_picture_convert(struct sna *sna,
 		FreePicture(dst, 0);
 		FreePicture(src, 0);
 
-		channel->bo = sna_pixmap_get_bo(tmp);
+		channel->bo = __sna_pixmap_get_bo(tmp);
 		kgem_bo_reference(channel->bo);
 		screen->DestroyPixmap(tmp);
 	} else {
@@ -2071,6 +2077,7 @@ sna_render_composite_redirect_done(struct sna *sna,
 						op->dst.pixmap->drawable.bitsPerPixel,
 						&t->box, 1);
 			assert(ok);
+			(void)ok;
 		}
 		if (t->damage) {
 			DBG(("%s: combining damage (all? %d), offset=(%d, %d)\n",
@@ -2105,7 +2112,7 @@ sna_render_copy_boxes__overlap(struct sna *sna, uint8_t alu,
 	if (tmp == NULL)
 		return false;
 
-	bo = sna_pixmap_get_bo(tmp);
+	bo = __sna_pixmap_get_bo(tmp);
 	if (bo == NULL)
 		goto out;
 
